<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\Base\Hook;

use IMIA\Compass\AssetBuilder;
use Assetic\AssetWriter;

/**
 * @package     imia_base
 * @subpackage  Hook
 * @author      David Frerich <d.frerich@imia.de>
 */
class TypoScriptFrontendController
{
    /**
     * @var array
     */
    protected $settings = array();

    /**
     */
    public function configArrayPostProc()
    {
        $this->settings = $this->parseTSSetup(
            (array)$GLOBALS['TSFE']->tmpl->setup['config.']['assetic.']['settings.']
        );
        $this->addAssets((array)$this->parseTSSetup($GLOBALS['TSFE']->tmpl->setup['config.']['assetic.'])); // global
        $this->addAssets((array)$this->parseTSSetup($GLOBALS['TSFE']->pSetup['assetic.'])); // page specific
    }

    /**
     * @param array $setup
     * @return array
     */
    protected function parseTSSetup($setup)
    {
        $config = array();
        foreach ($setup as $key => $value) {
            if (is_array($value)) {
                $value = $this->parseTSSetup($value);
            } elseif (is_string($value) && strpos($value, 'EXT:') === 0) {
                $value = preg_replace('/^EXT:[ ]*/', 'typo3conf/ext/', $value);
            }

            if (strrpos($key, '.') === strlen($key)-1) {
                $key = substr($key, 0, -1);
            }

            $config[$key] = $value;
        }

        return $config;
    }

    /**
     * @param array $config
     */
    public function addAssets($config)
    {
        if (array_key_exists('enabled', $config) && $config['enabled'] == 1) {
            unset($config['enabled']);
            foreach ($config as $name => $assetCollection) {
                if (is_array($assetCollection) && array_key_exists('output', $assetCollection)
                  && array_key_exists('enabled', $assetCollection) && $assetCollection['enabled']
                  && array_key_exists('files', $assetCollection) && is_array($assetCollection['files'])) {
                    $asseticCompile = false;

                    if (strtolower($_REQUEST['assetic']) == ($name)) {
                        $asseticCompile = true;
                    } elseif ($assetCollection['enabled'] == 2 && in_array(strtolower($_REQUEST['assetic']), array('all', 'compile'))) {
                        $asseticCompile = true;
                    } elseif ($assetCollection['enabled'] == 1) {
                        $time = filemtime(PATH_site . $assetCollection['output']);

                        foreach ($assetCollection['files'] as $file) {
                            if (strpos($file, '*')) {
                                $dir = substr($file, 0, strpos($file, '*'));
                                $ext = substr($file, strpos($file, '*')+1);

                                $handle = dir(PATH_site . $dir);
                                if ($handle) {
                                    while (false !== ($entry = $handle->read())) {
                                        if ($entry != '.' && $entry != '..') {
                                            if (preg_match('/' . preg_quote($ext) . '$/ism', $entry)) {
                                                if (filemtime(PATH_site . $dir . $entry) > $time) {
                                                    $asseticCompile = true;
                                                    break;
                                                }
                                            }
                                        }
                                    }
                                    $handle->close();
                                }
                            } else {
                                if (filemtime(PATH_site . $file) > $time) {
                                    $asseticCompile = true;
                                    break;
                                }
                            }
                        }
                    }

                    if ($asseticCompile) {
                        $this->createAssetCollection($assetCollection);
                    }

                    switch(substr($assetCollection['output'], strrpos($assetCollection['output'], '.'))) {
                        case '.js':
                            $javascriptTS = 'includeJS.';
                            if ($assetCollection['location'] == 'footer') {
                                $javascriptTS = 'includeJSFooter.';
                            }
                            $GLOBALS['TSFE']->pSetup[$javascriptTS][] = $assetCollection['output'];
                            break;
                        case '.css':
                        default:
                            $GLOBALS['TSFE']->pSetup['includeCSS.'][] = $assetCollection['output'];
                            break;
                    }
                }
            }
        }
    }

    /**
     * @param array $config
     */
    protected function createAssetCollection($config)
    {
        $settings = array(
            'rootPath'  => PATH_site,
            'cachePath' => PATH_site . 'typo3temp',
            'javaPath'  => $this->settings['javaPath'],
            'compile'   => array(
                'filters'   => array_map('trim', explode(',', $config['filters'])),
                'output'    => $config['output'],
            ),
            'filters'   => $this->getFilterSettings(),
        );

        $assetBuilder = new AssetBuilder($settings);
        $asset = $assetBuilder->build($config['files']);

        $this->ensureDirExists(dirname($settings['compile']['output']));
        $aw = new AssetWriter($settings['rootPath']);
        $aw->writeAsset($asset);
    }

    /**
     * @return array
     */
    protected function getFilterSettings()
    {
        $filterSettings = $this->settings;
        unset($filterSettings['javaPath']);

        if (array_key_exists('compass', $filterSettings) && array_key_exists('plugins', $filterSettings['compass'])) {
            $filterSettings['compass']['plugins'] = array_map('trim', explode(',', $filterSettings['compass']['plugins']));
        }

        return $filterSettings;
    }

    /**
     * @param string $dir
     */
    protected function ensureDirExists($dir)
    {
        if (!is_dir($dir)) {
            $this->ensureDirExists(dirname($dir));
            @mkdir($dir);
        }
    }
}